#-------------------------------------------------------------------------------
# Copyright 2009  Swiss Federal Institute of Aquatic Science and Technology
#
# This file is part of UWIM (Urban Water Infrastructure Model).
#
# UWIM is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# UWIM is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with UWIM.  If not, see <http://www.gnu.org/licenses/>.
#
# contact: max.maurer@eawag.ch
#===============================================================================
#
# Urban Water Infrastructure Model (UWIM), static version
#
# functions
# only SI-units are used
# Version 0.3.s
# 15.5.2009
#===============================================================================


#========================
# calculate the length of the sewer system
#======================== 

Laenge = function (P,Geom) {
       # P = list of parameter
       # Geom = output of Geometrie()

       #distance between sewer lines
       D.Ent.S=2*Geom$L.L               
      
       #exact number of sewer lines
       n.Ent.S=Geom$L.Q/D.Ent.S   
       
       #length of one pipesection the main sewer
       L.HS.H.eff=Geom$L.Q/round(n.Ent.S)+Geom$TK/round(n.Ent.S)     
       
       #length of one pipesection in a sewer line
       L.Ent.H.eff=Geom$B.Q/round(Geom$B.Q/P$L.Ent.H.est) 

       #number of pipesections in a sewer line
       n.Ent.H=round(Geom$B.Q/P$L.Ent.H.est)

       #number of pipesections above main sewer
       n.Ent.H1= min( max(round(n.Ent.H*P$Lage.HS),1),  n.Ent.H-1)
       
       #number of pipesections below main sewer
       n.Ent.H2=n.Ent.H-n.Ent.H1
       
       #total length of a sewer line
       L.Ent=(n.Ent.H-1) * L.Ent.H.eff * n.Ent.S

       #total length of the puplic sewer system
       L.tot.oeff=L.Ent + Geom$L.Q + Geom$TK
       
       #total length of the private sewer system    
       L.tot.privat=D.Ent.S*Geom$A / (4*Geom$A.L)
              
       #return
       return(list(
              D.Ent.S=D.Ent.S,            #distance between secundary lines 
              L.HS.H.eff=L.HS.H.eff,      #length of one pipesection of the main sewer
              n.Ent.S=n.Ent.S,            #exact number of sewer lines
              n.Ent.H=n.Ent.H,            #number of pipesections in a sewer line
              n.Ent.H1=n.Ent.H1,        	#number of pipesections above main sewer
              n.Ent.H2=n.Ent.H2,          #number of pipesections below main sewer
              L.Ent.H.eff=L.Ent.H.eff,    #length of one pipesection in a sewer line
              L.Ent=L.Ent,                #total length of a sewer line
              L.tot.oeff=L.tot.oeff,      #total length of the puplic sewer system
              L.tot.privat=L.tot.privat   #total length of the private sewer system  
              ))
}

#========================
# bore dimension
#======================== 
# subfunctions

#------- rain yield factor -------
# interpolat the values form SN 640 350 
# t= time
# T= recurrence time
# region = "Engadin / Wallis", "Alpen", "Jura", "Mittelland / Tessin N", 
#          "Voralpen", "Tessine S" 

F.i=function(t, T, Region) {

	
	
       if(T>20*365*24*3600) stop("T muss zwischen 15.768e6 und 630.72e6 Sekunden liegen!")
       if(T<0.5) 		stop("T muss zwischen 15.768e6 und 630.72e6 Sekunden liegen!")
       
       #tabele for a-values
       a.DF=data.frame("T"=c(0.5,1,2,5,10,20)*365*24*3600,
                       "Engadin.Wallis"=c(11, 12.38, 14.02, 16.42, 18.31, 20.21)/1000,
                       "Alpen"=c(14.64, 17.8, 21.11, 25.61, 29.10, 32.55)/1000,
                       "Jura"=c(17.14, 21.49, 25.92, 31.66, 36.07, 40.48)/1000,
                       "Mittelland.TessinN"=c(17.01, 23.61, 30.23, 39.02, 45.66, 52.29)/1000,
                       "Voralpen"=c(20.22, 28.60, 37.02, 48.33, 56.76, 67.21)/1000,
                       "TessineS"=c(34.41, 41.91, 49.54, 59.47, 66.81, 74.4)/1000)
                       
       #table for b-values
       b.DF=data.frame("T"=c(0.5,1,2,5,10,20)*365*24*3600,
                       "Engadin.Wallis"=c(.335, .248, .204, .173, .158, .148)*3600,
                       "Alpen"=c(.316, .263, .236, .215, .206, .198)*3600,
                       "Jura"=c(.2, .193, .191, .187, .186, .185)*3600,
                       "Mittelland.TessinN"=c(.2, .219, .231, .241, .247, .251)*3600,
                       "Voralpen"=c(.198, .224, .241, .257, .264, .284)*3600,
                       "TessineS"=c(.272, .268, .267, .264, .261, .261)*3600)
       
       #lineare approximation                 
       approx.a=approxfun(a.DF$T, a.DF[,Region])
       approx.b=approxfun(b.DF$T, b.DF[,Region])  
       
       a=approx.a(T)       
       b=approx.b(T)

       return(a/(t+b))
}

#------- drainage -------
# calculate the amount of drainage

F.Q.tw = function (D.Ent.S, L.Ent.H.eff, E, A, Q.E.spez, Stundenteiler, q.tw.min)  {
       q.tw=max( Q.E.spez/Stundenteiler, q.tw.min)
       return( q.tw * D.Ent.S * L.Ent.H.eff * E/A)
}

#------- stormwater -------
# calculate the amount of drainage

F.Q.R = function (D.Ent.S, L.Ent.H.eff, psi, i.t)  {
       return( psi * D.Ent.S* L.Ent.H.eff * i.t)
}

#------- diametern -------
# calculated the diameter based on the flow rate

F.Nennweite = function(Q, k.st, J, Auslastung=0.8) {
       #vector including all available diameters
       NW=c(.25, .35, .4, .5, .6, .8, 1, 1.2, 1.5, 2, 2.5, 3, 4, 6, 8)
              
       #maximum throughput for all diamters
       Q.voll=k.st*(NW/4)^(2/3)*sqrt(J/100)*pi/4*NW^2
       
       #take the smallest possible diameter with a load < 0.8
       a=min(which(Q<=Auslastung*Q.voll))
       if (a==Inf) stop("All available diameters are to small!")
       return(NW[a])                            
}

#------- calculate the flow time -------

F.tf = function (L.Ent.H, NW, J, k.st) {
       tf=L.Ent.H / (k.st*(NW/4)^(2/3)*sqrt(J/100))
       return(tf)
}

#------- bore dimension -------

Liste = function (P, Laeng) {  

       
       #data.frame for all pipesections above the main sewer 
       
       D.O=data.frame(NW=rep(NA, Laeng$n.Ent.H1),   
                     L.Ent.H.eff=rep(Laeng$L.Ent.H.eff, Laeng$n.Ent.H1),          #length of the pipesections
                     Q=rep(NA, Laeng$n.Ent.H1),                               	#drainage
                     Pos=paste("O", Laeng$n.Ent.H1:1, sep=""))                	#position (1 is  next to main sewer)                          
       D.O$Pos=as.character(D.O$Pos)
	       
       #data.frame for all mains below the main trunk sewer 
       D.U=data.frame(NW=rep(NA, Laeng$n.Ent.H2),   
                     L.Ent.H.eff=rep(Laeng$L.Ent.H.eff, Laeng$n.Ent.H2),        	#the first pipesection by the main sewer has the half length
                     Q=rep(NA, Laeng$n.Ent.H2),                          		#drainage
                     Pos=paste("U", Laeng$n.Ent.H2:1, sep=""))               	#position (1 is  next to main sewer) 
       D.U$Pos=as.character(D.U$Pos)
       
       #-- dimension the pipesections above the main sewer --
       
	#t0
       tcum=P$t0
       for (k in 1:nrow(D.O)) {
               
              #rain yield factor
              i=F.i(t=tcum, T=P$T, P$Region)
                 
              #total drainage
              Q.tw=F.Q.tw(Laeng$D.Ent.S, D.O$L.Ent.H.eff[k], P$E, P$A, P$Q.E.spez, P$Stundenteiler, P$q.tw.min)*k	 	#drainage
	       #effective area
      		Aw=sum( (Laeng$D.Ent.S*D.O$L.Ent.H.eff)[1:k] ) * P$psi
       	Q.R= Aw * i														#stormwater	
                                          
              #dimension
              D.O$Q[k]=Q.tw+Q.R                #total drainage       
              D.O$NW[k]=F.Nennweite(D.O$Q[k], P$k.st, P$J.Ent)

              #cumulated flow time
              tcum=tcum+F.tf(D.O$L.Ent.H.eff[k], D.O$NW[k], P$J.Ent, P$k.st) 

       }
       
       #calculate the load
       D.O$Load= D.O$Q / ( P$k.st*(D.O$NW/4)^(2/3)*sqrt(P$J.Ent/100)*pi/4*(D.O$NW)^2 )
       
       #-- dimension the pipesections below the main sewer --
       
       #t0
       tcum=P$t0
       for (k in 1:nrow(D.U)) {
               
              #rain yield factor
              i=F.i(t=tcum, T=P$T, P$Region)
                 
              #total drainage
              Q.tw=F.Q.tw(Laeng$D.Ent.S, D.U$L.Ent.H.eff[k], P$E, P$A, P$Q.E.spez, P$Stundenteiler, P$q.tw.min)*k 		#drainage
              #effective area
     		Aw=sum( (Laeng$D.Ent.S*D.U$L.Ent.H.eff)[1:k] )* P$psi
       	Q.R= Aw * i	      													#stormwater
		
              #dimension
              D.U$Q[k]=Q.tw+Q.R                #total drainage        
              D.U$NW[k]=F.Nennweite(D.U$Q[k], P$k.st, P$J.Ent)

              #cumulated flow time
              tcum=tcum+F.tf(D.U$L.Ent.H.eff[k], D.U$NW[k], P$J.Ent, P$k.st) 

       }
       
       #calculate the load
       D.U$Load = D.U$Q / ( P$k.st*(D.U$NW/4)^(2/3)*sqrt(P$J.Ent/100)*pi/4*(D.U$NW)^2 )

      #--- dimension of the main sewer ---
       
       #correction factor due to the rounded number of sewer lines
       KF=1+(Laeng$n.Ent.S-round(Laeng$n.Ent.S))/round(Laeng$n.Ent.S)

       #data.frame for all pipesections of the main sewer
       Q.HS=cumsum( rep(KF*(max(D.O$Q)+max(D.U$Q)),round(Laeng$n.Ent.S)) )+P$Q0

       D.HS=data.frame(NW=rep(NA, round(Laeng$n.Ent.S)),   
                     L.HS.H.eff=rep(Laeng$L.HS.H.eff, round(Laeng$n.Ent.S)),        #length of one pipesection
                     Q=Q.HS,
                     Pos=paste("H", 1:round(Laeng$n.Ent.S), sep=""))                #position
       D.HS$Pos=as.character(D.HS$Pos)
	              
       #dimension
       for (i in 1:nrow(D.HS)){      
              D.HS$NW[i]=F.Nennweite(D.HS$Q[i], P$k.st, P$J.HS)
       }
       
       #calculate the load
       D.HS$Load = D.HS$Q / ( P$k.st*(D.HS$NW/4)^(2/3)*sqrt(P$J.HS/100)*pi/4*(D.HS$NW)^2 )
       
       return(list("D.HS"=D.HS, "D.O"=D.O, "D.U"=D.U))
}


#========================
# costs per meter
#======================== 
# newdata= DF with depth and diameter

Kosten.m = function (newdata, P) {
       

       K.m=rep(NA, nrow(newdata))
       
       for (i in 1:nrow(newdata)) {
       	#above critical depth
              if(newdata$Tiefe[i]<=P$Grenztiefe) {
			#consider the per proportion under the roads
              	K.m[i]= ((1-P$p.st) * ( (P$m.a*newdata$NW[i]+P$n.a) * newdata$Tiefe[i] + P$f1.K*P$m.b*newdata$NW[i]+P$f2.K*P$n.b ) 		#costs in field
                     	+ P$p.st * ( (P$m.a*newdata$NW[i]+P$n.a) * newdata$Tiefe[i] + (P$m.b*newdata$NW[i]+P$n.b) ) )	      	#costs in roads
              }
              #below critical depth
              if(newdata$Tiefe[i]>P$Grenztiefe) {
                     K.m[i]= P$f.Db*(P$m.a*newdata$NW[i]+P$n.a)*newdata$Tiefe[i] + (P$m.b*newdata$NW[i]+P$n.b) + (1-P$f.Db)*(P$m.a*newdata$NW[i]+P$n.a)*P$Grenztiefe
              }
       
       }
       return(K.m)

}


#=========================
# total costs
#=========================
 
Kosten = function (P, NW, Laeng) {

      #-- pipesections of the sewer lines --     
      Ent=rbind(NW$D.O, NW$D.U)         
      #calculate the depth 
      Ent$Tiefe=P$D1*Ent$NW+P$D.cov
      
      #-- pipesections of the main sewer --     
      Haupt=NW$D.HS         
      #calculate the depth  
      Haupt$Tiefe=P$D1*Haupt$NW+P$D.cov

      #-- costs --   

      K.Ent=sum(Kosten.m(Ent, P)*Ent$L.Ent.H.eff)		#costs for ONE sewer line 
      K.Haupt=sum(Kosten.m(Haupt, P)*Haupt$L.HS.H.eff)  	#costs for the main sewer
      
      K.tot= K.Ent*Laeng$n.Ent.S + K.Haupt 			#total costs
      
      K.m = K.tot/sum(Ent$Laenge, Haupt$Laenge)			#costs per meter  
       
      return(list(Kosten.tot=K.tot, Kosten.m=K.m))     

}


#========================
# calculate the bore distribution
#======================== 

NW.Verteilung = function (Out.NW, n.Ent.S) {
	 
 	#vector including all available diameters
       alle=data.frame(Var1=c(.25, .35, .4, .5, .6, .8, 1, 1.2, 1.5, 2, 2.5, 3, 4, 6, 8))
      
       #length per diameter of the secundary sewer lines [m] 
       a=data.frame(table(c(Out.NW$D.O$NW, Out.NW$D.U$NW)))           
       a[nrow(a),2]=a[nrow(a),2]-1                           	#reduce length by one pipesection
       a[,2]=a[,2]*Out.NW$D.O[1,2]*n.Ent.S      			#length per diameter [m]	      

       #length per diameter of the main sewer [m]           
       b=data.frame(table(Out.NW$D.HS$NW)*Out.NW$D.HS[1,2])

       #merging
       M=merge(alle,a,by.x="Var1",by.y="Var1", all=T) 
       M=merge(M,b,by.x="Var1",by.y="Var1", all=T)   
    
       #length per diameter
       Laenge.D=rowSums(M[,2:3], na.rm=T)
       names(Laenge.D)=M[,1]

       #relativ distribution
       Laenge.D=Laenge.D/sum(Laenge.D)*100
	
	#return
	return(Laenge.D)
}	

