#-------------------------------------------------------------------------------
# Copyright 2009  Swiss Federal Institute of Aquatic Science and Technology
#
# This file is part of UWIM (Urban Water Infrastructure Model).
#
# UWIM is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# UWIM is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with UWIM.  If not, see <http://www.gnu.org/licenses/>.
#
# contact: max.maurer@eawag.ch
#===============================================================================
#
# Urban Water Infrastructure Model (UWIM), static version
# main file
#
# only SI-units are used
# Version 0.3.s
# 15.5.2009
#===============================================================================


setwd('//Eawag-nas1/swwdata/Herlyn/UWIM/Modell UWIM/R/UWIM statisch/')	#set work directory
											#all 3 files must be stored there
											# !!! use "/" instead "\" to seperate the path
									
source("UWIM_F_s.r")									#load the functions in the files
source("UWIM_Gebiet_F_s.r")                                                	#

#----------------------
# 0. define parameter
#----------------------   

# all parameter are necessary
					  
P=list(#--geometry
       A=220*1e4, 				# size of the catchment area [m2]
       f1=3,                              # shape factor of the catchment area 
       f2=1.2,                            # shape factor of the housing plots
       E=9000,				# number of inhabitants 
       Anz.G=1250,                        # number of buildings
       Lage.HS=.6,				# position of the main sewer; =1, everything above main sewer, =0 everything below main sewer
       L.Ent.H.est=35,			# estimated length of a pipesection in the sewer lines [m]
       TK=0,             			# total length of intercepting sewer [m]
       
       #--bore distribution
       T=10*365*24*3600,			# recurrence time [s] 
       t0=6*60,				# lead time [s]
       Region="Mittelland.TessinN",	# "Engadin.Wallis", "Alpen", "Jura", 
						# "Voralpen", "TessineS", "Mittelland.TessinN"
       Q.E.spez=200/(1000*24*3600),	# specific sewage coefficient  per inhabitants [m3*(E*s)^-1]
       q.tw.min=0.01/1000,			# minimal peak drainage [m3*(E*s)^-1]
       Stundenteiler=10*3600,	#?? # Stundenteiler [s]
       psi=0.18,				# runoff coefficient []
       k.st=85,				# strickler coefficient [m^(1/3)/s]
       J.Ent=2.5,				# gradient of the sewer lines[%]
       J.HS=2.5,				# gradient of the main sewer [%]
       Q0=0,					# extern inflow in the main sewer [m3/s]

       #--costs
       p.st=0.7,				# percentage of sections under roads
       D1=2,					# diameter influence on depth
       D.cov=2,				# minimal depth of coverage [m]
       Grenztiefe=7,				# critical depth [m]
       
       #above critical depth
	m.a=179.210,				# diameter related coefficient [CHF/m^3]
	m.b=1904.377,       			# diameter related coefficient [CHF/m^2]
	n.a=212.572036,                   	# diameter independent coefficient [CHF/m^2]
	n.b=-54.085155,                 	# fixed cost per meter [CHF/m]
       f1.K=0.846,                     	# reduction factors considering the reduced costs afield
       f2.K=8.196,             		# field construction compared to under street construction 
       #below critical depth
       f.Db=0.0723                     	# reduction factor due to different construction methods below the critical depth
       
       )


#----------------------
# 1. generate the sewer system 
#----------------------   

#generate the sewer system
Gebiet.gross=Gebiet.Gen(P)                      

#show size of the catchment area
P$A

#modify size of the catchment area
P$A=100*1e4

#generate a second sewer system of the smaller catchment area
Gebiet.klein=Gebiet.Gen(P)

#brief output
Gebiet.gross
Gebiet.klein

#comprehensive output
str(Gebiet.klein)

#access to the calculated lengths
Gebiet.klein$Laenge.oeff          			# total length of the public sewer system
Gebiet.gross$Kosten-Gebiet.klein$Kosten   	# cost difference of the two sewer systems

#plot the bore distribution
plot(Gebiet.gross)

#each generated sewer system saves its own parameter
Gebiet.gross$P					#list of all parameters
Gebiet.gross$P$A					#size of the catchment area


#----

